from django.conf import settings


LINK_ENABLED = getattr(settings, 'GIPSY_TOOLBAR_LINK_ENABLED', True)
LINK_CONTEXT_NAME = getattr(
    settings, 'GIPSY_TOOLBAR_LINK_CONTEXT_NAME', 'object')
# GIPSY_TOOLBAR_LINK_INCLUDED_MODELS
# define which models links to admin edit page. If `None` all of the models
# Example:
# GIPSY_TOOLBAR_LINK_INCLUDED_MODELS = ('auth.User',)

LINK_INCLUDED_MODELS = getattr(settings, 'GIPSY_TOOLBAR_LINK_INCLUDED_MODELS',
                               None)

VERSION_INDICATOR = getattr(settings, 'GIPSY_VERSION_INDICATOR', None)
# options are nav, user
VERSION_INDICATOR_LOCATION = getattr(settings,
                                     'GIPSY_VERSION_INDICATOR_LOCATION', 'nav')



from django.contrib import admin
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse

from ..admin import GipsyMenu, ChildrenInline
from .models import GipsyDashboardMenu


class ChildrenDashboardInline(ChildrenInline):
    model = GipsyDashboardMenu
    exclude = ('icon', 'content_type', )


class GipsyDashboardMenuAdmin(GipsyMenu):
    inlines = [ChildrenDashboardInline]
    exclude = ('url', 'parent', 'content_type', )

    def save_formset(self, request, form, formset, change):
        """
        Populates the content type before saving the inline objects.
        """
        super(GipsyDashboardMenuAdmin, self).save_formset(request, form, formset, change)
        for item in formset:
            if not 'id' in item.cleaned_data:
                continue
            instance = item.cleaned_data['id']
            if instance.url == item.cleaned_data['url']:
                continue

            instance.url = item.cleaned_data['url']
            url_args = instance.get_url_args()
            if url_args:
                i = 0
                while i < len(url_args):
                    url = None
                    try:
                        url = reverse('admin:%s_%s_add' % (url_args[i], url_args[i+1]))
                    except:
                        pass
                    if url:
                        try:
                            instance.content_type = ContentType.objects.get(app_label=url_args[i], model=url_args[i+1])
                            instance.save()
                        except ContentType.DoesNotExist:
                            pass
                    i += 1

admin.site.register(GipsyDashboardMenu, GipsyDashboardMenuAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
        ('dashboard', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='gipsydashboardmenu',
            name='content_type',
            field=models.ForeignKey(blank=True, to='contenttypes.ContentType', null=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='gipsydashboardmenu',
            name='icon',
            field=models.CharField(help_text='Font awesome class                                         http://fortawesome.github.io/Font-Awesome/icons/                                         ie: fa-circle', max_length=20, verbose_name='icon'),
            preserve_default=True,
        ),
    ]



class Widget(object):
    template = ''
    show_title = True
    title = ''
    grid = 4

    def __init__(self, **kwargs):
        for key in kwargs:
            if hasattr(self.__class__, key):
                setattr(self, key, kwargs[key])


class WidgetItems(Widget):
    items = []

    def __init__(self, items, **kwargs):
        self.items = items
        super(WidgetItems, self).__init__(**kwargs)


class WidgetModelList(WidgetItems):
    template = 'gipsy/dashboard/widgets/model_list.html'
    title = 'most recent items'
    grid = 6


class WidgetAdminLog(Widget):
    template = 'gipsy/dashboard/widgets/admin_log.html'
    title = 'recent admin actions'
    grid = 6


class WidgetMetricsList(WidgetItems):
    template = 'gipsy/dashboard/widgets/metrics_list.html'
    title = 'at a glance'
    grid = 4


class WidgetMetricsSingle(Widget):
    template = 'gipsy/dashboard/widgets/metrics_single.html'
    title = 'metrics single'
    grid = 4
    count = 0
    label = 'items'


class WidgetLineChart(Widget):
    template = 'gipsy/dashboard/widgets/line_chart.html'
    title = 'line chart'
    grid = 8
    labels = []
    values = []

    def __init__(self, labels, values, **kwargs):
        self.labels = labels
        self.values = values
        super(WidgetLineChart, self).__init__(**kwargs)


class WidgetMetricsEvolution(Widget):
    template = 'gipsy/dashboard/widgets/metrics_evolution.html'
    title = 'currently active users'
    label = 'active users'
    label_bottom = ''
    grid = 4
    result = 0
    difference = 0
    percentage = 0

    def __init__(self, result, previous_result, **kwargs):
        self.result = result
        self.difference = int(result) - int(previous_result)
        if previous_result == 0:
            self.percentage = 100
        else:
            self.percentage = (int(result) * 100) / int(previous_result)
        super(WidgetMetricsEvolution, self).__init__(**kwargs)



from datetime import date, datetime, timedelta

from django.core.cache import cache

from oauth2client.client import AccessTokenRefreshError

from .widgets import WidgetMetricsEvolution, WidgetLineChart
from gipsy.dashboard.services.google_analytics_connector import GoogleAnalyticsConnector


class WidgetGAEvolution(WidgetMetricsEvolution):
    cache_time = 3600

    def __init__(self, metrics, **kwargs):
        cache_name = 'gipsy.dashboard.widgets.widgets_google_analytics.' + metrics
        cached_objects = cache.get(cache_name)
        if cached_objects is None:
            ga_connector = GoogleAnalyticsConnector()
            query_date = str((datetime.now() - timedelta(days=1)).date())
            try:
                result = ga_connector.start_service()\
                    .query(start_date=query_date, end_date=query_date, metrics=metrics).execute()
                previous_date = str((datetime.now() - timedelta(days=2)).date())
                previous_result = ga_connector.start_service()\
                    .query(start_date=previous_date, end_date=previous_date, metrics=metrics).execute()
            except AccessTokenRefreshError:
                result = {}
                previous_result = {}

            try:
                result = result['rows'][0][0]
            except KeyError:
                result = 0
            try:
                previous_result = previous_result['rows'][0][0]
            except KeyError:
                previous_result = 0

            cached_objects = {'result': result, 'previous_result': previous_result}
            cache.set(cache_name, cached_objects, self.cache_time)

        super(WidgetGAEvolution, self)\
            .__init__(cached_objects['result'], cached_objects['previous_result'], **kwargs)


class WidgetGAPageViewsEvolution(WidgetGAEvolution):
    title = 'page views'
    label = 'page viewed yesterday'
    label_bottom = 'pages viewed difference'

    def __init__(self, **kwargs):
        super(WidgetGAPageViewsEvolution, self)\
            .__init__('ga:pageviews', **kwargs)


class WidgetGASessionsEvolution(WidgetGAEvolution):
    title = 'daily unique visitors'
    label = 'unique visitors yesterday'
    label_bottom = 'unique visitors difference'

    def __init__(self, **kwargs):
        super(WidgetGASessionsEvolution, self)\
            .__init__('ga:sessions', **kwargs)


class WidgetGALineChart(WidgetLineChart):
    title = 'our traffic overview'
    cache_time = 86400

    def __init__(self, **kwargs):
        labels = []
        values = {}
        values['sessions'] = {'label': 'Sessions', 'color': '#abc4e6', 'values': []}
        values['pageviews'] = {'label': 'Page views', 'color': '#ffffff', 'values': []}
        cache_name = 'gipsy.dashboard.widgets.widgets_google_analytics.WidgetGALineChart'
        cached_objects = cache.get(cache_name)
        if cached_objects is None:
            ga_connector = GoogleAnalyticsConnector()
            today = date.today()
            for num in range(0, 10):
                day_date = today - timedelta(days=num)
                day = day_date.strftime("%Y-%m-%d")
                labels.append(day_date.strftime("%m-%d-%Y"))
                try:
                    result = ga_connector.start_service()\
                        .query(start_date=day, end_date=day, metrics='ga:pageviews, ga:sessions').execute()
                except AccessTokenRefreshError:
                    result = {'totalsForAllResults': {'ga:sessions': 0, 'a:pageviews': 0}}

                values['sessions']['values'].append(result['totalsForAllResults']['ga:sessions'])
                values['pageviews']['values'].append(result['totalsForAllResults']['ga:pageviews'])
            values['sessions']['values'] = values['sessions']['values'][::-1]
            values['pageviews']['values'] = values['pageviews']['values'][::-1]
            cached_objects = {'labels': labels[::-1], 'values': [values['sessions'], values['pageviews']]}
            cache.set(cache_name, cached_objects, self.cache_time)
        self.labels = cached_objects['labels']
        self.values = cached_objects['values']



from django import forms
from gipsy.dashboard.settings import GIPSY_DASHBOARD_HAS_ROWS, GIPSY_DASHBOARD_DEFAULT_GRID


class Dashboard(object):
    '''
    This class is collecting the widgets setup in the settings.GIPSY_DASHBOARD
    constant.

    It is adding widget in rows in order to display them more efficiently
    in grids and responsive designs. You can disable this functionnality by
    setting settings.GIPSY_DASHBOARD_HAS_ROWS to False. You can also change the
    value of grid by changing settings.GIPSY_DASHBOARD_DEFAULT_GRID (default 12).

    If you want to customize the look of your dashboard and it's modules, you
    can declare css stylesheets and/or javascript files to include when
    rendering the dashboard (these files should be placed in your
    media path), for example::

        from admin_tools.dashboard import Dashboard

        class MyDashboard(Dashboard):
            class Media:
                css = {
                    'all': (
                        'css/mydashboard.css',
                        'css/mystyles.css',
                    ),
                }
                js = (
                    'js/mydashboard.js',
                    'js/myscript.js',
                )
    '''
    # Using Django's Media meta class
    __metaclass__ = forms.MediaDefiningClass

    def _media(self):
        return forms.Media()
    media = property(_media)
    has_rows = True
    rows = [[]]
    widgets = []
    request = None

    def __init__(self, request):
        self.request = request
        self.widgets = []
        self.has_rows = GIPSY_DASHBOARD_HAS_ROWS
        self.rows = [[]]
        self.render()
        if self.has_rows:
            total = 0
            for widget in self.widgets:
                total += widget.grid
                if total > GIPSY_DASHBOARD_DEFAULT_GRID:
                    self.rows.append([])
                    total = widget.grid
                self.rows[-1].append(widget)

    def render(self):
        pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='GipsyToolbarMenu',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('url', models.CharField(blank=True, max_length=255, null=True, verbose_name='Url')),
                ('order', models.PositiveSmallIntegerField(verbose_name='Order')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE,
                 to='toolbar.GipsyToolbarMenu')),
            ],
            options={
                'ordering': ['order'],
                'abstract': False,
            },
        ),
    ]



from gipsy.dashboard.dashboard import Dashboard
from gipsy.dashboard.widgets import widgets, widgets_google_analytics


class DashboardDefault(Dashboard):
    """
    Defaut and exemple class for dashboards using google analytics specific widgets.
    This class simply uses a render method where widgets are added. The widgets property is a list
    that can be appended as follow.
    The google analytics widgets are plug and play but of course feel free to overwrite them.
    Read the documentation for more information on how to setup google analytics on yourapplication.
    """
    def render(self):
        # metrics evolution
        self.widgets.append(widgets_google_analytics.WidgetGAPageViewsEvolution())

        # metrics evolution
        self.widgets.append(widgets_google_analytics.WidgetGASessionsEvolution())

        # metrics single
        self.widgets.append(widgets.WidgetMetricsSingle(
            title='currently active users',
            label='active users',
            count=2564,
        ))

        # line chart
        self.widgets.append(widgets_google_analytics.WidgetGALineChart())

        # metrics list
        self.widgets.append(widgets.WidgetMetricsList(items=[
            {'icon': 'fa-file-image-o', 'label': 'posts', 'value': 75},
            {'icon': 'fa-comment-o', 'label': 'comments', 'value': 192},
            {'icon': 'fa-files-o', 'label': 'pages', 'value': 12},
            {'icon': 'fa-flag-o', 'label': 'in moderation', 'value': 4},
        ]))

        # model list
        self.widgets.append(widgets.WidgetModelList(items={}))

        # admin logs
        self.widgets.append(widgets.WidgetAdminLog())



from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _

from ..models import GipsyMenu


class GipsyDashboardMenu(GipsyMenu):
    icon = models.CharField(_('icon'), max_length=20,
                            help_text=_("Font awesome class \
                                        http://fortawesome.github.io/Font-Awesome/icons/\
                                         ie: fa-circle"))
    content_type = models.ForeignKey(ContentType, null=True, blank=True)
    _url_args = None
    _original_url = None

    def __init__(self, *args, **kwargs):
        super(GipsyDashboardMenu, self).__init__(*args, **kwargs)
        self._original_url = self.url

    @property
    def children(self):
        return GipsyDashboardMenu.objects.filter(parent=self.pk)

    def get_url_args(self):
        """
        Get the arguments from the url.
        """
        if self._url_args is None:
            if self.url is None:
                self._url_args = []
                return self._url_args
            url = self.url
            if url[0] != '/':
                url = '/' + url
            self._url_args = url.split('/')
        return self._url_args



from django.conf import settings
if 'gipsy.dashboard' in settings.INSTALLED_APPS:
    from gipsy.dashboard.admin import admin
else:
    from django.contrib import admin

from ..admin import GipsyMenu, ChildrenInline
from .models import GipsyToolbarMenu


class ChildrenToolbarInline(ChildrenInline):
    model = GipsyToolbarMenu


class GipsyToolbarMenuAdmin(GipsyMenu):
    inlines = [ChildrenToolbarInline]

admin.site.register(GipsyToolbarMenu, GipsyToolbarMenuAdmin)



import httplib2
from apiclient.discovery import build
from oauth2client.client import flow_from_clientsecrets
from oauth2client.file import Storage
from oauth2client.service_account import ServiceAccountCredentials
from oauth2client.tools import run_flow

from django.conf import settings


class GoogleAnalyticsConnector(object):
    """
    Connector class for the google API.
    You could use the auth method described in http://blog.iambob.me/accessing-google-analytics-from-django/
    and in the official API documentation
    https://developers.google.com/analytics/solutions/articles/hello-analytics-api
    Also you could use the type of authenticate credentials "Service account"
    https://developers.google.com/analytics/devguides/reporting/core/v3/quickstart/service-py
    For using this method you must define in settings.py follow lines, e.g.:
        GOOGLE_ANALYTICS_CREDENTIAL_TYPE = "oauth2_client_id"  # default value if it's not defined
        GOOGLE_ANALYTICS_TOKEN_FILE_NAME = "path/to/your/analytics.dat"
        GOOGLE_ANALYTICS_CLIENT_SECRETS = "path/to/your/client_secret.json"
        GOOGLE_ANALYTICS_VIEW_ID = "your_view_id"
    or
        GOOGLE_ANALYTICS_CREDENTIAL_TYPE = "service_account"
        GOOGLE_ANALYTICS_PRIVATE_KEY_FILE_NAME = "path/to/privatekey.p12/or/privatekey.pem"
        GOOGLE_ANALYTICS_CLIENT_EMAIL = "your-email-address@developer.gserviceaccount.com"
        GOOGLE_ANALYTICS_VIEW_ID = "your_view_id"
    If you will choose "service_account" credential type, don't forget
    to add your client email "your-email-address@developer.gserviceaccount.com"
    to Google Analytics. See more here:
    http://stackoverflow.com/questions/12837748/analytics-google-api-error-403-user-does-not-have-any-google-analytics-account
    """

    def __init__(self):
        self.service = None
        self.VIEW_ID = settings.GOOGLE_ANALYTICS_VIEW_ID
        self.SCOPE = 'https://www.googleapis.com/auth/analytics.readonly'

    def _credential_type(self):
        if hasattr(settings, 'GOOGLE_ANALYTICS_CREDENTIAL_TYPE'):
            return settings.GOOGLE_ANALYTICS_CREDENTIAL_TYPE
        return "oauth2_client_id"

    def _authenticate(self):
        credentials = None
        if "service_account" == self._credential_type():
            primary_key_file = settings.GOOGLE_ANALYTICS_PRIVATE_KEY_FILE_NAME
            with open(primary_key_file, 'rb') as f:
                private_key = f.read()
            credentials = ServiceAccountCredentials.from_p12_keyfil(
                settings.GOOGLE_ANALYTICS_CLIENT_EMAIL, private_key, self.SCOPE)
        else:
            storage = Storage(settings.GOOGLE_ANALYTICS_TOKEN_FILE_NAME)
            credentials = storage.get()
            if credentials is None or credentials.invalid:
                flow = flow_from_clientsecrets(
                    settings.GOOGLE_ANALYTICS_CLIENT_SECRETS, scope=self.SCOPE)
                credentials = run_flow(flow, storage, self.GAFlags())
        return credentials

    def _create_service(self):
        # 1. Create an http object
        http = httplib2.Http()

        # 2. Authorize the http object
        credentials = self._authenticate()
        http = credentials.authorize(http)

        # 3. Build the Analytics Service Object with the authorized http object
        return build('analytics', 'v3', http=http)

    def start_service(self):
        """
        :return: ``this`` Google analytics object with the service set

        start the service which may be used to query the google analytics api
        """
        if not self.service:
            self.service = self._create_service()
        return self

    def query(self, **kwargs):
        kwargs.update({
            'ids': "ga:{id}".format(id=self.VIEW_ID),
        })

        result = self.service.data().ga().get(**kwargs)
        return result

    class GAFlags():
        """
        total hack. If you want to see why, examine apiclient.sample_tools.
        The python bindings as well as the documentation kind've forgot to
        mention this... Which is a big deal... But actually though.
        """
        noauth_local_webserver = True
        logging_level = "DEBUG"
